.wp-block[data-align=center] .wp-block-search .wp-block-search__inside-wrapper {
  margin: auto;
}

.wp-block-search :where(.wp-block-search__button) {
  height: auto;
  border-radius: initial;
  display: flex;
  align-items: center;
  justify-content: center;
  text-align: center;
}

.wp-block-search__inspector-controls .components-base-control {
  margin-bottom: 0;
}