/**
 * Colors
 */
.wp-block[data-align=center] > .wp-block-button {
  text-align: center;
  margin-left: auto;
  margin-right: auto;
}

.wp-block[data-align=right] > .wp-block-button {
  /*!rtl:ignore*/
  text-align: right;
}

.wp-block-button {
  position: relative;
  cursor: text;
}
.wp-block-button:focus {
  box-shadow: 0 0 0 1px #fff, 0 0 0 3px var(--wp-admin-theme-color);
  outline: 2px solid transparent;
  outline-offset: -2px;
}
.wp-block-button[data-rich-text-placeholder]::after {
  opacity: 0.8;
}

div[data-type="core/button"] {
  display: table;
}